@@ -23,163 +23,50 @@ For more information about SPF, a tool against email forgery, see
     http://www.openspf.org/
 
 For news, bugfixes, etc. visit the home page for this implementation at
-    http://www.wayforward.net/spf/
+    http://cheeseshop.python.org/pypi/pyspf/
     http://sourceforge.net/projects/pymilter/
+    http://www.wayforward.net/spf/
 """
 
-# Changes:
-#    9-dec-2003, v1.1, Meng Weng Wong added PTR code, THANK YOU
-#   11-dec-2003, v1.2, ttw added macro expansion, exp=, and redirect=
-#   13-dec-2003, v1.3, ttw added %{o} original domain macro,
-#                      print spf result on command line, support default=,
-#                      support localhost, follow DNS CNAMEs, cache DNS results
-#                      during query, support Python 2.2 for Mac OS X
-#   16-dec-2003, v1.4, ttw fixed include handling (include is a mechanism,
-#                      complete with status results, so -include: should work.
-#                      Expand macros AFTER looking for status characters ?-+
-#                      so altavista.com SPF records work.
-#   17-dec-2003, v1.5, ttw use socket.inet_aton() instead of DNS.addr2bin, so
-#                      n, n.n, and n.n.n forms for IPv4 addresses work, and to
-#                      ditch the annoying Python 2.4 FutureWarning
-#   18-dec-2003, v1.6, Failures on Intel hardware: endianness.  Use ! on
-#                      struct.pack(), struct.unpack().
-#
-# Development taken over by Stuart Gathman <stuart@bmsi.com>.
-#
+# CVS Commits since last release (2.0.1):
 # $Log: spf.py,v $
-# Revision 1.108  2006/11/08 01:27:00  customdesigned
-# Return all key-value-pairs in Received-SPF header for all results.
+# Revision 1.108.2.6  2006/12/30 17:12:50  customdesigned
+# Merge fixes from CVS HEAD.
 #
-# Revision 1.107  2006/11/04 21:58:12  customdesigned
-# Prevent cache poisoning by bogus additional RRs in PTR DNS response.
+# Revision 1.108.2.5  2006/12/24 19:10:38  kitterma
+# Move spf.py changelog to CHANGELOG. Move spf.py cvs commits from previous
+# releases to py_spfchangelog.txt. Update README to describe provided scripts.
+# Add to README discussion of spf module interface.
 #
-# Revision 1.106  2006/10/16 20:48:24  customdesigned
-# More DOS limit tests.
+# Revision 1.108.2.4  2006/12/23 06:35:37  customdesigned
+# Fully quote structured values in Received-SPF.
 #
-# Revision 1.105  2006/10/07 22:06:28  kitterma
-# Pass strict status to DNSLookup - will be needed for TCP failover.
+# Revision 1.108.2.3  2006/12/23 04:44:05  customdesigned
+# Fix key-value quoting in get_header.
 #
-# Revision 1.104  2006/10/07 21:59:37  customdesigned
-# long/empty label tests and fix.
+# Revision 1.121  2006/12/30 17:01:52  customdesigned
+# Missed a spot for new result names.
 #
-# Revision 1.103  2006/10/07 18:16:20  customdesigned
-# Add tests for and fix RE_TOPLAB.
+# Revision 1.120  2006/12/28 04:54:21  customdesigned
+# Skip optional trailing ";" in Received-SPF
 #
-# Revision 1.102  2006/10/05 13:57:15  customdesigned
-# Remove isSPF and make missing space after version tag a warning.
+# Revision 1.118  2006/12/28 04:04:27  customdesigned
+# Optimize get_header to remove useless key-value pairs.
 #
-# Revision 1.101  2006/10/05 13:39:11  customdesigned
-# SPF version tag is case insensitive.
+# Revision 1.117  2006/12/23 06:31:16  customdesigned
+# Fully quote values in key-value pairs.
 #
-# Revision 1.100  2006/10/04 02:14:04  customdesigned
-# Remove incomplete saving of result.  Was messing up bmsmilter.  Would
-# be useful if done consistently - and disabled when passing spf= to check().
+# Revision 1.108.2.2  2006/12/22 20:27:24  customdesigned
+# Index error reporting non-mech permerror.
 #
-# Revision 1.99  2006/10/03 21:00:26  customdesigned
-# Correct fat fingered merge error.
+# Revision 1.108.2.1  2006/12/22 04:59:40  customdesigned
+# Merge comma heuristic.
 #
-# Revision 1.98  2006/10/03 17:35:45  customdesigned
-# Provide python inet_ntop and inet_pton when not socket.has_ipv6
-#
-# Revision 1.97  2006/10/02 17:10:13  customdesigned
-# Test and fix for uppercase macros.
-#
-# Revision 1.96  2006/10/01 01:27:54  customdesigned
-# Switch to pymilter lax processing convention:
-# Always return strict result, extended result in q.perm_error.ext
-#
-# Revision 1.95  2006/09/30 22:53:44  customdesigned
-# Fix getp to obey SHOULDs in RFC.
-#
-# Revision 1.94  2006/09/30 22:23:25  customdesigned
-# p macro tests and fixes
-#
-# Revision 1.93  2006/09/30 20:57:06  customdesigned
-# Remove generator expression for compatibility with python2.3.
-#
-# Revision 1.92  2006/09/30 19:52:52  customdesigned
-# Removed redundant flag and unneeded global.
-#
-# Revision 1.91  2006/09/30 19:37:49  customdesigned
-# Missing L
-#
-# Revision 1.90  2006/09/30 19:29:58  customdesigned
-# pydns returns AAAA RR as binary string
-#
-# Revision 1.89  2006/09/29 20:23:11  customdesigned
-# Optimize cidrmatch
-#
-# Revision 1.88  2006/09/29 19:44:10  customdesigned
-# Fix ptr with ip6 for harsh mode.
-#
-# Revision 1.87  2006/09/29 19:26:53  customdesigned
-# Add PTR tests and fix ip6 ptr
-#
-# Revision 1.86  2006/09/29 17:55:22  customdesigned
-# Pass ip6 tests
-#
-# Revision 1.85  2006/09/29 15:58:02  customdesigned
-# Pass self test on non IP6 python.
-# PTR accepts no cidr.
-#
-# Revision 1.83  2006/09/27 18:09:40  kitterma
-# Converted spf.check to return pre-MARID result codes for drop in
-# compatibility with pySPF 1.6/1.7.  Added new procedure, spf.check2 to
-# return RFC4408 results in a two part answer (result, explanation).
-# This is the external API for pySPF 2.0.  No longer any need to branch
-# for 'classic' and RFC compliant pySPF libraries.
-#
-# Revision 1.82  2006/09/27 18:02:21  kitterma
-# Converted max MX limit to ambiguity warning for validator.
-#
-# Revision 1.81  2006/09/27 17:38:14  kitterma
-# Updated initial comments and moved pre-1.7 changes to spf_changelog.
-#
-# Revision 1.80  2006/09/27 17:33:53  kitterma
-# Fixed indentation error in check0.
-#
-# Revision 1.79  2006/09/26 18:05:44  kitterma
-# Removed unused receiver policy definitions.
-#
-# Revision 1.78  2006/09/26 16:15:50  kitterma
-# added additional IP4 and CIDR validation tests - no code changes.
-#
-# Revision 1.77  2006/09/25 19:42:32  customdesigned
-# Fix unknown macro sentinel
-#
-# Revision 1.76  2006/09/25 19:10:40  customdesigned
-# Fix exp= error and add another failing test.
-#
-# Revision 1.75  2006/09/25 02:02:30  kitterma
-# Fixed redirect-cancels-exp test suite failure.
-#
-# Revision 1.74  2006/09/24 04:04:08  kitterma
-# Implemented check for macro 'c' - Macro unimplimented.
-#
-# Revision 1.73  2006/09/24 02:08:35  kitterma
-# Fixed invalid-macro-char test failure.
-#
-# Revision 1.72  2006/09/23 05:45:52  kitterma
-# Fixed domain-name-truncation test failure
-#
-# Revision 1.71  2006/09/22 01:02:54  kitterma
-# pySPF correction for nolocalpart in rfc4408-tests.yml failed, 4.3/2.
-# Added comments to testspf.py on where to get YAML.
-#
-# Revision 1.70  2006/09/18 02:13:27  kitterma
-# Worked through a large number of pylint issues - all 4 spaces, not a mix
-# of 4 spaces, 2 spaces, and tabs. Caught a few minor errors in the process.
-# All built in tests still pass.
-#
-# Revision 1.69  2006/09/17 18:44:25  kitterma
-# Fixed validation mode only crash bug when rDNS check had no PTR record
-#
-#
-# See spf_changelog.txt for earlier changes.
+# See pyspf_changelog.txt for earlier CVS commits.
 
 __author__ = "Terence Way"
 __email__ = "terry@wayforward.net"
-__version__ = "1.7: July 22, 2005"
+__version__ = "2.0.2: December 23, 2006"
 MODULE = 'spf'
 
 USAGE = """To check an incoming mail request:
@@ -248,6 +135,9 @@ RE_TOPLAB = re.compile(
     r'\.(?:[0-9a-z]*[a-z][0-9a-z]*|[0-9a-z]+-[0-9a-z-]*[0-9a-z])\.?$|%s'
     	% PAT_CHAR, re.IGNORECASE)
 
+RE_DOT_ATOM = re.compile(r'%(atext)s+([.]%(atext)s+)*$' % {
+    'atext': r"[0-9a-z!#$%&'*+/=?^_`{}|~-]" }, re.IGNORECASE)
+
 RE_IP6 = re.compile(                 '(?:%(hex4)s:){6}%(ls32)s$'
                    '|::(?:%(hex4)s:){5}%(ls32)s$'
                   '|(?:%(hex4)s)?::(?:%(hex4)s:){4}%(ls32)s$'
@@ -269,9 +159,9 @@ JOINERS = {'l': '.', 's': '.'}
 
 RESULTS = {'+': 'pass', '-': 'fail', '?': 'neutral', '~': 'softfail',
            'pass': 'pass', 'fail': 'fail', 'permerror': 'permerror',
-       'error': 'error', 'neutral': 'neutral', 'softfail': 'softfail',
+       'error': 'temperror', 'neutral': 'neutral', 'softfail': 'softfail',
        'none': 'none', 'local': 'local', 'trusted': 'trusted',
-           'ambiguous': 'ambiguous'}
+           'ambiguous': 'ambiguous', 'unknown': 'permerror' }
 
 EXPLANATIONS = {'pass': 'sender SPF authorized',
                 'fail': 'SPF fail - not authorized',
@@ -689,7 +579,14 @@ class query(object):
 
     >>> q.validate_mechanism('a:mail.example.com.')
     ('a:mail.example.com.', 'a', 'mail.example.com', 32, 'pass')
-        """
+
+    >>> try: q.validate_mechanism('a:mail.example.com,')
+    ... except PermError,x: print x
+    Do not separate mechnisms with commas: a:mail.example.com,
+    """
+        if mech.endswith( "," ):
+            self.note_error('Do not separate mechnisms with commas', mech)
+	    mech = mech[:-1]
         # a mechanism
         m, arg, cidrlength, cidr6length = parse_mechanism(mech, self.d)
         # map '?' '+' or '-' to 'neutral' 'pass' or 'fail'
@@ -1247,16 +1144,24 @@ class query(object):
     def get_header(self, res, receiver=None):
         if not receiver:
             receiver = self.r
-        if res == 'permerror':
+	client_ip = self.c
+	helo = quote_value(self.h)
+	if self.ident == 'helo':
+	    envelope_from = None
+	else:
+	    envelope_from = quote_value(self.s)
+        if res == 'permerror' and self.mech:
             tag = ' '.join([res] + self.mech)
-	    return '%s (%s: %s) client-ip=%s; envelope-from=%s; helo=%s; ' \
-	    	   'receiver=%s; identity=%s; problem=%s;' % (
-		tag, receiver, self.get_header_comment(res), self.c,
-		self.l + '@' + self.o, self.h, receiver, self.ident, self.mech)
-	return '%s (%s: %s) client-ip=%s; envelope-from=%s; helo=%s; ' \
-		'receiver=%s; identity=%s;' % (
-	    res, receiver, self.get_header_comment(res), self.c,
-	    self.l + '@' + self.o, self.h, receiver, self.ident)
+	    problem = quote_value(' '.join(self.mech))
+	else:
+	    tag = res
+	    problem = None
+	res = ['%s (%s: %s)' % (tag,receiver,self.get_header_comment(res))]
+	for k in ('client_ip','envelope_from','helo','receiver','problem'):
+	    v = locals()[k]
+	    if v: res.append('%s=%s;'%(k,v))
+	res.append('identity=%s'%self.ident)
+	return ' '.join(res)
 
     def get_header_comment(self, res):
         """Return comment for Received-SPF header.
@@ -1279,7 +1184,7 @@ class query(object):
         elif res == 'permerror': return \
             "permanent error in processing domain of %s: %s" \
                   % (sender, self.prob)
-        elif res == 'error': return \
+        elif res == 'temperror': return \
               "temporary error in processing during lookup of %s" % sender
         elif res == 'fail': return \
               "domain of %s does not designate %s as permitted sender" \
@@ -1311,6 +1216,33 @@ def split_email(s, h):
         else:
             return 'postmaster', s
 
+def quote_value(s):
+    """Quote the value for a key-value pair in Received-SPF header field
+    if needed.  No quoting needed for a dot-atom value.
+
+    Examples:
+    >>> quote_value('foo@bar.com')
+    '"foo@bar.com"'
+    
+    >>> quote_value('mail.example.com')
+    'mail.example.com'
+
+    >>> quote_value('A:1.2.3.4')
+    '"A:1.2.3.4"'
+
+    >>> quote_value('abc"def')
+    '"abc\\\\"def"'
+
+    >>> quote_value(r'abc\def')
+    '"abc\\\\\\\\def"'
+
+    >>> quote_value('abc..def')
+    '"abc..def"'
+    """
+    if RE_DOT_ATOM.match(s):
+      return s
+    return '"' + s.replace('\\',r'\\').replace('"',r'\"') + '"'
+
 def parse_mechanism(str, d):
     """Breaks A, MX, IP4, and PTR mechanisms into a (name, domain,
     cidr,cidr6) tuple.  The domain portion defaults to d if not present,
@@ -1326,8 +1258,8 @@ def parse_mechanism(str, d):
     >>> parse_mechanism('a/24', 'foo.com')
     ('a', 'foo.com', 24, None)
 
-    >>> parse_mechanism('A:foo:bar.com/16', 'foo.com')
-    ('a', 'foo:bar.com', 16, None)
+    >>> parse_mechanism('A:foo:bar.com/16//48', 'foo.com')
+    ('a', 'foo:bar.com', 16, 48)
 
     >>> parse_mechanism('-exists:%{i}.%{s1}.100/86400.rate.%{d}','foo.com')
     ('-exists', '%{i}.%{s1}.100/86400.rate.%{d}', None, None)
@@ -1335,8 +1267,8 @@ def parse_mechanism(str, d):
     >>> parse_mechanism('mx:%%%_/.Claranet.de/27','foo.com')
     ('mx', '%%%_/.Claranet.de', 27, None)
 
-    >>> parse_mechanism('mx:%{d}/27','foo.com')
-    ('mx', '%{d}', 27, None)
+    >>> parse_mechanism('mx:%{d}//97','foo.com')
+    ('mx', '%{d}', None, 97)
 
     >>> parse_mechanism('iP4:192.0.0.0/8','foo.com')
     ('ip4', '192.0.0.0', 8, None)
